.include "nes.h"
.include "fizzbuzz.h"
.include "mbyt.h"

.segment "ZEROPAGE"
pagenum: .res 1
pagemovedir: .res 1  ; 0: going to next; 1: going to previous
pagesrcaddr: .res 2
pagesrccolor: .res 1

.segment "CODE"
.proc show_help
ydst = $08
paldst = $09
xscroll_lo = $0A
xscroll_hi = $0B

  lda #SFX_FIZZBUZZ
  jsr start_sound
  lda #0
  sta pagenum
  sta pageprogress
  sta pagemovedir

helploop:
  lda #$FF
  sta ydst
  sta paldst

  ; progress frames
  ; 0-2: Load lines 1-3
  ; 3: Load faded palette
  ; 4-6: Load lines 4-6
  ; 12: Load original palette
  ; Draw even pages (0, ...) to second nametable and odd to first!
  ldy pageprogress
  bne notprogress0
  sty pagesrccolor
  lda pagenum
  asl a
  tay
  lda help_pages,y
  sta pagesrcaddr
  lda help_pages+1,y
  sta pagesrcaddr+1
  ldy #0
notprogress0:
  cpy #16
  bcs noprogressinc
  cpy #7
  bcs no_loadline
  cpy #3
  beq no_loadline
  bcs :+
  iny
:
  ; render even pages to the odd nametable
  lda pagenum
  lsr a
  tya
  bcs :+
  ora #8
:
  sta ydst

  jsr fizz_clrline
  ldx #2
  lday pagesrcaddr
  stay 0
  lda pagesrccolor
  sta 2
  jsr fizz_puts_color0
  stay pagesrcaddr
  lda 2
  sta pagesrccolor
  jmp loadline_done

no_loadline:
  cpy #3
  bne :+
  lda #$10
  bne have_paldst
:
  cpy #12
  bne loadline_done
  lda #$00
have_paldst:
  sta paldst

loadline_done:
  inc pageprogress
noprogressinc:

  ; Translate progress to a scroll position
  lda pageprogress
  cmp #8
  bcc :+
  adc #111 ; 0-16 -> 0-7,120-128
:
  asl a
  sta xscroll_lo
  lda #0
  rol a
  bit pagemovedir
  bpl :+
  sta xscroll_hi
  lda #1  ; 1 to compensate for carry clear from rol
  sbc xscroll_lo
  sta xscroll_lo
  lda #2
  sbc xscroll_hi
:
  eor pagenum
  and #$01
  ora #VBLANK_NMI
  sta xscroll_hi
  
  lda nmis
:
  cmp nmis
  beq :-
  ldy ydst
  bmi otherthancopy
  jsr fizz_copyline_wide
  jmp no_copyline
otherthancopy:
  lda paldst
  bmi no_copyline
  jsr copy_faded_pal
no_copyline:
  ldx xscroll_lo
  lda xscroll_hi
  ldy #8

  clc
  jsr ppu_screen_on
  jsr read_pads
  jsr update_sound

  lda pageprogress
  cmp #14
  bcc notLeft

  lda new_keys
  and #KEY_START
  bne helpdone1

  lda new_keys
  and #KEY_A|KEY_RIGHT
  beq notRight
  lda pagenum
  cmp #NUM_HELP_PAGES-1
  bcs lastRight
  inc pagenum
  lda #0
  sta pageprogress
  sta pagemovedir
  lda #SFX_COUNT
  jsr start_sound
  jmp notRight
lastRight:
  lda new_keys
  bmi helpdone1
notRight:

  lda new_keys
  and #KEY_LEFT
  beq notLeft
  lda pagenum
  beq notLeft
  dec pagenum
  ldy #0
  sty pageprogress
  dey
  sty pagemovedir
  lda #SFX_COUNT
  jsr start_sound
notLeft:

  jmp helploop

helpdone1:
  lda #SFX_FIZZBUZZ
  jsr start_sound
  rts
.endproc

;;
;
.proc copy_faded_pal
  eor #$FF
  tay
  iny
  ldx #$3F
  stx PPUADDR
  ldx #0
  stx PPUADDR
loop:
  tya
  clc
  adc initial_palette,x
  bpl have_A
  lda #$0F  ; map negative colors to black
have_A:
  sta PPUDATA
  inx
  cpx #12
  bcc loop
  rts
.endproc

.segment "RODATA"
initial_palette:
  mbyt "0f001020 0f1a2a20 0f172720"

help_pages:
  .addr version_page
  .addr help_page1, help_page2, help_page3, help_page4, help_page5
  .addr help_page6, help_page7, help_page8, help_page9, help_page10
  .addr help_page11
NUM_HELP_PAGES = (* - help_pages) / 2

hello_msg:
pregame_help_msg:
  .byte 2,"B: fizz 3n",1,10
  .byte "multiples of 3",10
  .byte 3,"A: buzz 5n",1,10
  .byte "multiples of 5",10
  .byte "down: count",10
  .byte "other numbers",0,0

BUILDDAY = (.TIME / 86400) - 16405
version_page:
  .byte "nes ",2,"fizz",3,"buzz ",1,10
  .byte .sprintf("w.i.p. day %d", BUILDDAY),10
  .byte "copr. 2014",10
  .byte "damian yerrick",10,10
  .byte "press right",0
help_page1:
  .byte "this is a game",10
  .byte "of counting to",10
  .byte "a hundred, but",10
  .byte "with a twist.",0
help_page2:
  .byte "for multiples",10
  .byte "of 3, like 3, 6,",10
  .byte "or 21, you ",2,"fizz",1,10
  .byte "instead of",10
  .byte "counting.",10
  .byte "1, 2, ",2,"fizz...",0
help_page3:
  .byte "for multiples",10
  .byte "of 5, like 5, 10,",10
  .byte "or 20, ",3,"buzz.",1,10
  .byte "4, ",3,"buzz, ",2,"fizz, ",1,"7...",0
help_page4:
  .byte "some numbers",10
  .byte "are multiples",10
  .byte "of both 3 and 5,",10
  .byte "like 15 or 30.",10
  .byte "for these,",10
  .byte "you ",2,"fizz",3,"buzz.",1,0
help_page5:
  .byte "down: count",10
  .byte "B: ",2,"fizz   ",1,"A: ",3,"buzz",1,10
  .byte 10
  .byte "get one wrong",10
  .byte "and you'll have",10
  .byte "to wait a while.",0
help_page6:
  .byte "watch out!",10
  .byte "later rounds",10
  .byte "change when to",10
  .byte 2,"fizz ",1,"and ",3,"buzz.",1,10
  .byte "for example:",0
help_page7:
  .byte 2,"fizz 7n ",1,10
  .byte "multiples of 7",10
  .byte "these are",10
  .byte "7, 14, 21, 28, ...",0
help_page8:
  .byte 3,"buzz 60/n",1,10
  .byte "factors of 60",10
  .byte "these are",10
  .byte "1-6, 10, 12, 15,",10
  .byte "20, 30, and 60.",0
help_page9:
  .byte 2,"fizz prime",1,10
  .byte "only 2 factors",10
  .byte "these are",10
  .byte "2, 3, 5, 7, 11, 13,",10
  .byte "17, 19, 23, 29, ...",0
help_page10:
  .byte 3,"buzz n",$84,1,10
  .byte "perfect square",10
  .byte "these are",10
  .byte "1, 4, 9, 16, 25,",10
  .byte "36, 49, 64, 81,",10
  .byte "and 100.",0
help_page11:
  .byte 2,"fizz fibo",1,10
  .byte "sum of last 2",10
  .byte "these are",10
  .byte "1, 2, 3, 5, 8,",10
  .byte "13, 21, 34, 55,",10
  .byte "and 89.",0,0

